import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:hub/shared/helpers/secure_storage_helper.dart';
import 'package:hub/shared/helpers/shared_preferences_storage_helper.dart';
import 'package:hub/shared/helpers/sqlite_storage_helper.dart';
import 'package:hub/shared/utils/cache_util.dart';
import 'package:shared_preferences/shared_preferences.dart';

enum StorageType { SecureStorage, SharedPreferences, SQLite3 }

abstract class Storage {
  Future<void> set(String key, dynamic value, Function(dynamic) cacheSetter);
  Future<dynamic> get(String key);
  Future<void> delete(String key);
  Future<void> purge();
}

class StorageData {
  final String key;
  final dynamic value;
  final StorageType type;

  StorageData(this.key, this.value, this.type);
}

class StorageUtil {
  static final StorageUtil _instance = StorageUtil._internal();

  factory StorageUtil() {
    return _instance;
  }

  StorageUtil._internal();

  final SQLiteStorageHelper _sqliteStorage = SQLiteStorageHelper();
  final SecureStorageHelper _secureStorage = SecureStorageHelper();
  final SharedPreferencesStorageHelper _sharedPreferences = SharedPreferencesStorageHelper();

  static Future<void> set(StorageData data) async {
    try {
      switch (data.type) {
        case StorageType.SecureStorage:
          await SecureStorageHelper().set(data.key, data.value, (v){});
          break;
        case StorageType.SharedPreferences:
          await SharedPreferencesStorageHelper().set(data.key, data.value, (v){});
          break;
        case StorageType.SQLite3:
          await SQLiteStorageHelper().set(data.key, data.value, (v){});
          break;
      }
    } catch (e) {
      log('Error setting data: $e');
    }
  }

  static Future<dynamic> get(StorageData data) async {
    try {
      var value = CacheUtil().get(data.key);
      if (value == null) {
        switch (data.type) {
          case StorageType.SecureStorage:
            value = await SecureStorageHelper().get(data.key);
            break;
          case StorageType.SharedPreferences:
            value = await SharedPreferencesStorageHelper().get(data.key);
            break;
          case StorageType.SQLite3:
            value = await SQLiteStorageHelper().get(data.key);
            break;
        }
        CacheUtil().set(data.key, value);
      }
      return value;
    } catch (e) {
      log('Error getting data: $e');
      return null;
    }
  }

  static Future<void> delete(StorageData data) async {
    try {
      CacheUtil().delete(data.key);
      switch (data.type) {
        case StorageType.SecureStorage:
          await SecureStorageHelper().delete(data.key);
          break;
        case StorageType.SharedPreferences:
          await SharedPreferencesStorageHelper().delete(data.key);
          break;
        case StorageType.SQLite3:
          await SQLiteStorageHelper().delete(data.key);
          break;
      }
    } catch (e) {
      log('Error deleting data: $e');
    }
  }

  static Future<void> purge() async {
    try {
      await SecureStorageHelper().purge();
      await SharedPreferencesStorageHelper().purge();
      await SQLiteStorageHelper().purge();
    } catch (e) {
      log('Error purging data: $e');
    }
  }

  Future<void> ensureInitialization() async {
    await initSharedPreferences();
    await initSecureStorage();
    await initSQLiteStorage();
  }

  Future<void> initSharedPreferences() async {
    if (_sharedPreferences.prefs == null) {
      _sharedPreferences.prefs = await SharedPreferences.getInstance();
      _sharedPreferences.isFirstRun = _sharedPreferences.prefs?.getBool('first_run') ?? true;
    }
    if (_sharedPreferences.isFirstRun) {
      _sharedPreferences.isFirstRun = false;
      _sharedPreferences.purge();
    }
  }

  Future<void> initSecureStorage() async {
    log('SecureStorageHelper: Starting initialization');
    try {
      _email = await _secureStorage.getString('ff_email');
      _passwd = await _secureStorage.getString('ff_passwd');
      _deviceType = await _secureStorage.getString('ff_deviceType');
      _isLogged = await _secureStorage.getBool('ff_isLogged');
      _tokenAPNS = await _secureStorage.getString('ff_tokenAPNS');
      _accessPass = await _secureStorage.getString('accessPass');
      _panicPass = await _secureStorage.getString('panicPass');
      _fingerprintPass = await _secureStorage.getString('fingerprintPass');
      _haveLocal = await _secureStorage.getBool('ff_have_local');
      _deviceDescription = await _secureStorage.getString('deviceDescription');
    } catch (e) {
      log('SecureStorageHelper: Error during initialization: $e');
    }
    log('SecureStorageHelper: Initialization complete');
  }

  Future<void> initSQLiteStorage() async {
    log('SQLiteStorageHelper: Starting initialization');
    try {
      await _sqliteStorage.database;
    } catch (e) {
      log('SQLiteStorageHelper: Error during initialization: $e');
    }
    log('SQLiteStorageHelper: Initialization complete');
  }

  bool _isFirstRun = true;
  bool get isFirstRun => _isFirstRun;
  set isFirstRun(bool value) {
    _isFirstRun = value;
    _sharedPreferences.set('first_run', value, (v){});
  }

  String? _deviceDescription;
  String get deviceDescription => _deviceDescription ?? '';
  set deviceDescription(String value) => _secureStorage.set('deviceDescription', value, (v) => _deviceDescription = v);

  BuildContext? _context;
  BuildContext? get context => _context;
  set context(BuildContext? value) => _secureStorage.set('ff_context', value.toString(), (v) => _context = value);

  bool? _haveLocal;
  bool? get haveLocal => _haveLocal;
  set haveLocal(bool? value) => _secureStorage.set('ff_have_local', value ?? false, (v) => _haveLocal = value);

  String? _fingerprintPass;
  String get fingerprintPass => _fingerprintPass ?? '';
  set fingerprintPass(String value) => _secureStorage.set('fingerprintPass', value, (v) => _fingerprintPass = v);

  String? _accessPass;
  String get accessPass => _accessPass ?? '';
  set accessPass(String value) => _secureStorage.set('accessPass', value, (v) => _accessPass = v);

  String? _panicPass;
  String get panicPass => _panicPass ?? '';
  set panicPass(String value) => _secureStorage.set('panicPass', value, (v) => _panicPass = v);

  String? _tokenAPNS;
  String? get tokenAPNS => _tokenAPNS;
  set tokenAPNS(String? value) => _secureStorage.set('ff_tokenAPNS', value ?? '', (v) => _tokenAPNS = v);

  String? _email;
  String get email => _email ?? '';
  set email(String value) => _secureStorage.set('ff_email', value, (v) => _email = v);

  String? _passwd;
  String get passwd => _passwd ?? '';
  set passwd(String value) => _secureStorage.set('ff_passwd', value, (v) => _passwd = v);

  String? _deviceType;
  String get deviceType => _deviceType ?? '';
  set deviceType(String value) => _secureStorage.set('ff_deviceType', value, (v) => _deviceType = v);

  bool? _isLogged;
  bool get isLogged => _isLogged ?? false;
  set isLogged(bool value) => _secureStorage.set('ff_isLogged', value, (v) => _isLogged = v);

  String? _token;
  String get token => _token ?? '';
  set token(String value) => _secureStorage.set('ff_token', value, (v) => _token = v);


  final String _devUUID = '';
  String get devUUID => _devUUID;
  set devUUID(String value) => _sqliteStorage.set('devUUID', value, (v){});

  final String _userUUID = '';
  String get userUUID => _userUUID;
  set userUUID(String value) => _sqliteStorage.set('userUUID', value, (v){});

  final String _userDevUUID = '';
  String get userDevUUID => _userDevUUID;
  set userDevUUID(String value) => _sqliteStorage.set('userDevUUID', value, (v){});

  final String _status = '';
  String get status => _status;
  set status(String value) => _sqliteStorage.set('status', value, (v){});

  final String _userName = '';
  String get userName => _userName;
  set userName(String value) => _sqliteStorage.set('userName', value, (v){});

  final String _cliUUID = '';
  String get cliUUID => _cliUUID;
  set cliUUID(String value) => _sqliteStorage.set('cliUUID', value, (v){});

  final String _ownerUUID = '';
  String get ownerUUID => _ownerUUID;
  set ownerUUID(String value) => _sqliteStorage.set('ownerUUID', value, (v){});

  final String _cliName = '';
  String get cliName => _cliName;
  set cliName(String value) => _sqliteStorage.set('cliName', value, (v){});

  final String _petAmountRegister = '';
  String get petAmountRegister => _petAmountRegister;
  set petAmountRegister(String value) => _sqliteStorage.set('petAmountRegister', value, (v){});

  final bool _whatsapp = false;
  bool get whatsapp => _whatsapp;
  set whatsapp(bool value) => _sqliteStorage.set('whatsapp', value, (v){});

  final bool _provisional = false;
  bool get provisional => _provisional;
  set provisional(bool value) => _sqliteStorage.set('provisional', value, (v){});

  final bool _pets = false;
  bool get pets => _pets;
  set pets(bool value) => _sqliteStorage.set('pets', value, (v){});

  final bool _local = false;
  bool get local => _local;
  set local(bool value) => _sqliteStorage.set('local', value, (v){});

  final bool _notify = false;
  bool get notify => _notify;
  set notify(bool value) => _sqliteStorage.set('notify', value, (v){});

  final bool _fingerprint = false;
  bool get fingerprint => _fingerprint;
  set fingerprint(bool value) => _sqliteStorage.set('fingerprint', value, (v){});

  final bool _access = false;
  bool get access => _access;
  set access(bool value) => _sqliteStorage.set('access', value, (v){});

  final bool _panic = false;
  bool get panic => _panic;
  set panic(bool value) => _sqliteStorage.set('panic', value, (v){});

  final bool _person = false;
  bool get person => _person;
  set person(bool value) => _sqliteStorage.set('person', value, (v){});

  final bool _requestOSnotification = false;
  bool get requestOSnotification => _requestOSnotification;
  set requestOSnotification(bool value) => _sqliteStorage.set('requestOSnotification', value, (v){});

  String get createdAt => '0000-00-00 00:00:00';
}
